#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <ctype.h>

extern char *optarg;

#define M_TWOPI 6.28318530717958647692 // 2 * Pi

#define RESULTSFILE "results.txt"

static int quiet = 0;

int pvstrlwr (char *a)
{
	int i;

	for(i=0; i< strlen(a); i++) a[i] = tolower(a[i]);
}

int pvstrcmpi(const char *a, const char *b)
{

	while(tolower(*a) == tolower(*b)) { a++; b++; if (!*a && !*b) return 0; }
	return 1;
}

///// Useful Stuff From <rnd.c> /////

double sq(const double x)
{
	return x*x;
}

int Maxi(const int a,const int b)
{
	return (a>b)?a:b;
}

int Mini(const int a,const int b)
{
	return (a<b)?a:b;
}

double frnd(const double lim)
{
	return (double)rand()/RAND_MAX*lim;
}

double Max(const double a,const double b)
{
	return (a>b)?a:b;
}

int rnd(const int lim)
{
	return (int)frnd((double)lim-0.0001);
}

///// Text I/O /////

char timefmt_ret[10][100]; int timefmt_bank=0;

char *timefmt(const double t)
{
	char *str=timefmt_ret[timefmt_bank=(timefmt_bank+1)%10];
	if (t<1e-6) sprintf(str,"%.1lf ns",t*1e9);
	else if (t<1e-3) sprintf(str,"%.2lf us",t*1e6);
	else sprintf(str,"%.3lf ms",t*1e3);
	return str;
}

void report(const char *s)
{
	if(!quiet) printf("%s\n",s);
}

///// Particle Structure /////

enum PM {Pion=0,Muon=1}; // Particle species 
enum ST {Active=0,Stopped=1,Backwards=2,Reimpact=3,Finished=-1}; 


typedef struct
{
	double x,y,z,vx,vy,vz; // Position, velocity vectors
	enum PM s; // Particle species
	enum ST state;
	double m0,wt,t,t_decay; // Rest mass, weight (from Mont Carlo input), proper time, proper time of decay
	int ns1,ns2,ns3; // Three nearby solenoids
} particle;

typedef struct
{
	double x,y,z,vx,vy,vz;
} V6;

particle operator +(register particle a,register const V6 b)
{
	a.x+=b.x; a.y+=b.y; a.z+=b.z;
	a.vx+=b.vx; a.vy+=b.vy; a.vz+=b.vz;
	return a;
}

particle addhalf(register particle a,register const V6 b)
{
	a.x+=b.x/2; a.y+=b.y/2; a.z+=b.z/2;
	a.vx+=b.vx/2; a.vy+=b.vy/2; a.vz+=b.vz/2;
	return a;
}

///// Relativity Utils /////

#define eV 1.602176462e-19
#define c 2.99792458e8

double beta_from_gamma(const double g)
{
	return sqrt(1.0-1.0/g/g);
}

double gamma_from_xyz(register const double vx,register const double vy,register const double vz)
{
	return 1.0/sqrt(1.0-(vx*vx+vy*vy+vz*vz)/c/c);
}

double invgamma_from_xyz(register const double vx,register const double vy,register const double vz)
{
	return sqrt(1.0-(vx*vx+vy*vy+vz*vz)/c/c);
}

double ungamma_from_xyz(register const double vx,register const double vy,register const double vz)
{
	return 1.0/sqrt(1.0+(vx*vx+vy*vy+vz*vz)/c/c);
}

double gammav_from_gamma(const double g)
{
	return c*sqrt(g*g-1);
}

///// Force Functions /////

#define E0_mu 1.692833322785576416e-11 // 105.6583568e6*eV
#define m0_mu 1.883531091486804744e-28 // E0_mu/c/c
#define E0_pi 2.2361576880134e-11 // (134.9764e6+4.5936e6=139.57e6)*eV
#define m0_pi 2.488060976912839308e-28 // E0_pi/c/c

typedef struct
{
	double x,y,z,ax,ay,az;
	double r,l,strength;
	double ux,uy,uz;
} solenoid;

solenoid sol[100]; int msol=0;

double bz3(const double zR,const double LzR,const double rR) // Inputs z/R, (L-z)/R, r/R
{
	double q1,q2,s1,s2;
	if (rR>1.2) return 0;
	q1=1.0/(zR*zR+1); s1=sqrt(q1);
	q2=1.0/(LzR*LzR+1); s2=sqrt(q2);
	return 0.5*(zR*s1+LzR*s2+0.75*rR*rR*(zR*s1*q1*q1+LzR*s2*q2*q2));
}

double br3(const double zR,const double LzR,const double rR)
{
	double q1,q2,t1,t2;
	if (rR>1.2) return 0;
	q1=1.0/(zR*zR+1); t1=sqrt(q1)*q1;
	q2=1.0/(LzR*LzR+1); t2=sqrt(q2)*q2;
	return 0.25*rR*(t2-t1+0.375*rR*rR*((zR*zR*4-1)*t1*q1*q1-(LzR*LzR*4-1)*t2*q2*q2));
}

solenoid makesolenoid(const double x1,const double y1,const double z1,
	const double x2,const double y2,const double z2,const double r,const double strength)
{
	solenoid s;
	s.x=x1; s.y=y1; s.z=z1;
	s.ax=x2-x1; s.ay=y2-y1; s.az=z2-z1;
	s.r=r; s.l=sqrt(sq(x2-x1)+sq(y2-y1)+sq(z2-z1)); s.strength=strength;
	s.ux=s.ax/s.l; s.uy=s.ay/s.l; s.uz=s.az/s.l;
	return s;
}

char addsolenoid(const double x1,const double y1,const double z1,
	const double x2,const double y2,const double z2,const double r,const double strength)
{
	if (msol<100)
	{
		sol[msol]=makesolenoid(x1,y1,z1,x2,y2,z2,r,strength);
		msol++;
		return 1;
	}
	return 0;
}

void findsolenoids(particle *p)
{
	int n; double dd1,dd2,dd3,dd;
	if (msol<=3) return;
	p->ns1=0; dd1=sq(p->x-sol[0].x-sol[0].ax/2)+sq(p->y-sol[0].y-sol[0].ay/2)+sq(p->z-sol[0].z-sol[0].az/2);
	p->ns2=1; dd2=sq(p->x-sol[1].x-sol[1].ax/2)+sq(p->y-sol[1].y-sol[1].ay/2)+sq(p->z-sol[1].z-sol[1].az/2);
	p->ns3=2; dd3=sq(p->x-sol[2].x-sol[2].ax/2)+sq(p->y-sol[2].y-sol[2].ay/2)+sq(p->z-sol[2].z-sol[2].az/2);
	for (n=3;n<msol;n++)
	{
		dd=sq(p->x-sol[n].x-sol[n].ax/2)+sq(p->y-sol[n].y-sol[n].ay/2)+sq(p->z-sol[n].z-sol[n].az/2);
		if (dd<dd1 || dd<dd2 || dd<dd3)
		{
			if (dd1>=dd2 && dd1>=dd3) {dd1=dd; p->ns1=n;}
			else if (dd2>=dd1 && dd2>=dd3) {dd2=dd; p->ns2=n;}
			else {dd3=dd; p->ns3=n;}
		}
	}
}

void addsolfield(const solenoid s,double px,double py,double pz,double *bx,double *by,double *bz)
{
	double z,r,cz,cr,L=s.l,R=s.r;
	px-=s.x; py-=s.y; pz-=s.z;
	z=px*s.ux+py*s.uy+pz*s.uz;
	px-=z*s.ux; py-=z*s.uy; pz-=z*s.uz;
	r=sqrt(px*px+py*py+pz*pz);
	cz=bz3(z/R,(L-z)/R,r/R);
	cr=br3(z/R,(L-z)/R,r/R);
	px/=r; py/=r; pz/=r;
	*bx+=(s.ux*cz+px*cr)*s.strength;
	*by+=(s.uy*cz+py*cr)*s.strength;
	*bz+=(s.uz*cz+pz*cr)*s.strength;
}

V6 fdt(const double t,const particle p,const double dt) // d(x,y,z,vx,vy,vz)/dt
{
	int n; V6 ret;
	double Bx=0,By=0,Bz=0,
		gamma=gamma_from_xyz(p.vx,p.vy,p.vz),
		v2=p.vx*p.vx+p.vy*p.vy+p.vz*p.vz,
		nudtoverm0=dt*c/sqrt(c*c+gamma*gamma*v2)/p.m0,
		r=-1.0/(sq(c/gamma)+v2),fx,fy,fz,rvdotf;
	if (msol<=3) for (n=0;n<msol;n++) addsolfield(sol[n],p.x,p.y,p.z,&Bx,&By,&Bz);
	else
	{
		addsolfield(sol[p.ns1],p.x,p.y,p.z,&Bx,&By,&Bz);
		addsolfield(sol[p.ns2],p.x,p.y,p.z,&Bx,&By,&Bz);
		addsolfield(sol[p.ns3],p.x,p.y,p.z,&Bx,&By,&Bz);
	}
	fx=eV*(p.vy*Bz-p.vz*By);
	fy=eV*(p.vz*Bx-p.vx*Bz);
	fz=eV*(p.vx*By-p.vy*Bx);
	rvdotf=r*(p.vx*fx+p.vy*fy+p.vz*fz);
	ret.x=p.vx*dt; ret.y=p.vy*dt; ret.z=p.vz*dt;
	ret.vx=(p.vx*rvdotf+fx)*nudtoverm0;
	ret.vy=(p.vy*rvdotf+fy)*nudtoverm0;
	ret.vz=(p.vz*rvdotf+fz)*nudtoverm0;
	return ret;
}

///// Beam Operations /////

void savebeam(const char *filename,const particle *p,const int m)
{
	int n; FILE *out;
	char *str=(char *)malloc(strlen(filename)+6);
	report("Saving beam...");
	strcpy(str,filename);
	for (n=strlen(str)-1;str[n]!='.' && n>0;n--);
	if (n==0) n=strlen(str);
	strcpy(str+n,".beam");
	out=fopen(str,"wb");
	free(str);
	fwrite(&m,sizeof(int),1,out);
	fwrite(p,sizeof(particle),m,out);
	fclose(out);
	report("Beam saved");
}

int loadbeam(const char *filename,particle **p,int *m) // *p should either be already allocated or entirely NULL
{
	int n; FILE *in;
	char *str=(char *)malloc(strlen(filename)+6);
	report("Loading beam...");
	strcpy(str,filename);
	for (n=strlen(str)-1;str[n]!='.' && n>0;n--);
	if (n==0) n=strlen(str);
	strcpy(str+n,".beam");
	if (in=fopen(str,"rb"))
	{
		particle *pn,*pm;
		free(str);
		fread(m,sizeof(int),1,in);
		if (*p) free(*p);
		fread(*p=(particle *)malloc(*m*sizeof(particle)),sizeof(particle),*m,in);
		fclose(in);
		pm=*p+*m;
		for (pn=*p;pn<pm;pn++) findsolenoids(pn); // What it says
		report("Beam loaded");
		return 1;
	}
	else {report("Load failed"); free(str); return 0;}
}

#define RSRC 0.01
#define LSRC 0.20

double ZSRC=0.0,ROTSRC=0.200;

int loadbeam_rodsource(const char *filename,particle **p,int *m) // *p should either be already allocated or entirely NULL
{
	int n; FILE *in;
	char *str=(char *)malloc(Maxi(200,strlen(filename)+5));
	report("Loading tantalum rod source data...");
	strcpy(str,filename);
	for (n=strlen(str)-1;str[n]!='.' && n>0;n--);
	if (n==0) n=strlen(str);
	strcpy(str+n,".csv");
#ifdef VMS
	if (in=fopen(str,"r")) // Load pions
#else
	if (in=fopen(str,"rt")) // Load pions
#endif
	{
		double cr=cos(ROTSRC),sr=sin(ROTSRC);
		particle *pn,*pm;
		*m=-1;
		while (!feof(in) && fgets(str,200,in)) if (strlen(str)>1) (*m)++;
		//*m/=10; // Decimate beam
		if (*p) free(*p);
		*p=(particle *)malloc(*m*sizeof(particle));
		pm=*p+*m;
		rewind(in); fgets(str,200,in);
		for (n=0,pn=*p;pn<pm;pn++,n++)
		{
			double ke,wt,vx,vy,vz,mallards,x,y,z;
			if (n%1000==0)
			{
				sprintf(str,"Reading CSV file... %d / %d",n,*m);
				report(str);
			}
			fscanf(in,"2,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,,%lf,%lf,%lf\n",
				&ke,&wt,&vx,&vy,&vz,&mallards,&mallards,&mallards,&x,&y,&z);
			z-=LSRC/2*100.0;
			pn->x=0.01*(x*cr+z*sr);
			pn->y=0.01*y;
			pn->z=ZSRC+0.01*(z*cr-x*sr);
			ke=c*beta_from_gamma(ke*1e6*eV/E0_pi+1.0);
			pn->vx=ke*(vx*cr+vz*sr);
			pn->vy=ke*vy;
			pn->vz=ke*(vz*cr-vx*sr);
			pn->s=Pion; pn->state=Active; pn->m0=m0_pi; pn->wt=wt;
			pn->t=0; pn->t_decay=-log(frnd(1))*2.6033e-8;
		}
		for (pn=*p;pn<pm;pn++) findsolenoids(pn); // What it says
		fclose(in); free(str);
		report("CSV read complete");
		return 1;
	}
	else
	{
		free(str);
		report("CSV read failed");
		return 0;
	}
}

double diff_infinitynorm(const char *file1,const char *file2)
{
	int n;
	particle *p1=NULL,*p2=NULL; int m1,m2,mm;
	double ret=0;
	if (!loadbeam(file1,&p1,&m1)) return -1;
	if (!loadbeam(file2,&p2,&m2)) {free(p1); return -2;}
	mm=Mini(m1,m2);
	for (n=0;n<mm;n++)
	{
		if (p1[n].z>0)
			ret=Max(ret,sqrt(sq(p1[n].x-p2[n].x)+sq(p1[n].y-p2[n].y)+sq(p1[n].z-p2[n].z)));
		if (n%100==0)
		{
			char str[100];
			sprintf(str,"Comparing beams [L-infinity]... %d / %d",n,mm);
			report(str);
		}
	}
	free(p1); free(p2);
	report("L-infinity comparison complete");
	return ret;
}

double diff_rms(const char *file1,const char *file2)
{
	int n;
	particle *p1=NULL,*p2=NULL; int m1,m2,mm,m;
	double ret=0;
	if (!loadbeam(file1,&p1,&m1)) return -1;
	if (!loadbeam(file2,&p2,&m2)) {free(p1); return -2;}
	mm=Mini(m1,m2);
	for (n=m=0;n<mm;n++)
	{
		if (p1[n].z>0)
		{
			ret+=sq(p1[n].x-p2[n].x)+sq(p1[n].y-p2[n].y)+sq(p1[n].z-p2[n].z);
			m++;
		}
		if (n%1000==0)
		{
			char str[100];
			sprintf(str,"Comparing beams [RMS]... %d / %d",n,mm);
			report(str);
		}
	}
	if (m>0) ret=sqrt(ret/m); else return -3;
	free(p1); free(p2);
	report("RMS comparison complete");
	return ret;
}

double diff_mean(const char *file1,const char *file2)
{
	int n;
	particle *p1=NULL,*p2=NULL; int m1,m2,mm,m;
	double ret=0;
	if (!loadbeam(file1,&p1,&m1)) return -1;
	if (!loadbeam(file2,&p2,&m2)) {free(p1); return -2;}
	mm=Mini(m1,m2);
	for (n=m=0;n<mm;n++)
	{
		if (p1[n].z>0)
		{
			ret+=sqrt(sq(p1[n].x-p2[n].x)+sq(p1[n].y-p2[n].y)+sq(p1[n].z-p2[n].z));
			m++;
		}
		if (n%1000==0)
		{
			char str[100];
			sprintf(str,"Comparing beams [average]... %d / %d",n,mm);
			report(str);
		}
	}
	if (m>0) ret/=m; else return -3;
	free(p1); free(p2);
	report("Average comparison complete");
	return ret;
}

///// Calculation Routine /////

double t,cROTSRC,sROTSRC;

void advanceparticle(register particle *pn,register const double dt)
{
	int n;
	register V6 h1=fdt(t,*pn,dt),
		h2=fdt(t+dt/2,addhalf(*pn,h1),dt),
		h3=fdt(t+dt/2,addhalf(*pn,h2),dt),
		h4=fdt(t+dt,*pn+h3,dt);
	pn->x+=(h1.x+h2.x+h2.x+h3.x+h3.x+h4.x)/6;
	pn->y+=(h1.y+h2.y+h2.y+h3.y+h3.y+h4.y)/6;
	pn->z+=(h1.z+h2.z+h2.z+h3.z+h3.z+h4.z)/6;
	pn->vx+=(h1.vx+h2.vx+h2.vx+h3.vx+h3.vx+h4.vx)/6;
	pn->vy+=(h1.vy+h2.vy+h2.vy+h3.vy+h3.vy+h4.vy)/6;
	pn->vz+=(h1.vz+h2.vz+h2.vz+h3.vz+h3.vz+h4.vz)/6;
	pn->t+=dt*invgamma_from_xyz(pn->vx,pn->vy,pn->vz);
	if (pn->s==Pion && pn->t>=pn->t_decay) // pi+ to mu+, nu_mu decay
	{
		register double dx,dy,dz,dd,gamma=gamma_from_xyz(pn->vx,pn->vy,pn->vz);
		do {dx=frnd(2)-1; dy=frnd(2)-1; dz=frnd(2)-1;}
		while ((dd=dx*dx+dy*dy+dz*dz)>1);
		dd=29.792e6*eV/c/m0_mu/sqrt(dd);
		dx*=dd; dy*=dd; dz*=dd;
		pn->vx*=gamma; pn->vy*=gamma; pn->vz*=gamma;
		pn->vx+=dx; pn->vy+=dy; pn->vz+=dz;
		gamma=ungamma_from_xyz(pn->vx,pn->vy,pn->vz);
		pn->vx*=gamma; pn->vy*=gamma; pn->vz*=gamma;
		pn->s=Muon; pn->m0=m0_mu;
	}
	if (t>0.025e-9) if (sq(pn->x*cROTSRC-(pn->z-ZSRC)*sROTSRC)+sq(pn->y)<RSRC*RSRC*0.99 &&
		(pn->z-ZSRC)*cROTSRC+pn->x*sROTSRC>LSRC*-0.495 &&
		(pn->z-ZSRC)*cROTSRC+pn->x*sROTSRC<LSRC*0.495) pn->state=Reimpact;
	for (n=0;n<msol;n++)
	{
		double x=pn->x-sol[n].x,y=pn->y-sol[n].y,z=pn->z-sol[n].z,dp=x*sol[n].ux+y*sol[n].uy+z*sol[n].uz;
		if (dp>=0 && dp<=sol[n].l)
		{
			x-=dp*sol[n].ux; y-=dp*sol[n].uy; z-=dp*sol[n].uz; dp=(x*x+y*y+z*z)/sq(sol[n].r);
			if (dp>=1 && dp<120) pn->state=Stopped;
		}
	}
	if (pn->z<-0.4) pn->state=Backwards;
	if (pn->z>31) pn->state=Finished;
}

///// Genetic Algorithm Module /////

#define GRES 1000

typedef struct
{
	int m;
	char **name;
	int *param;
} genome;

genome newgenome()
{
	genome ret;
	ret.m=0;
	return ret;
}

void removegenome(genome *g)
{
	int n;
	for (n=0;n<g->m;n++) free(g->name[n]);
	free(g->name);
	free(g->param);
	g->m=0;
}

void gassign(genome *g,const char *s,const int v)
{
	int n,sa=-1;
	for (n=0;n<g->m && sa<0;n++) if (!pvstrcmpi(g->name[n],s)) sa=n;
	if (sa>=0) g->param[sa]=v;
	else // New variable
	{
		if (g->m)
		{
			char **ct=(char **)malloc((g->m+1)*sizeof(char *)),*st=(char *)malloc(strlen(s)+1);
			int *it=(int *)malloc((g->m+1)*sizeof(int));
			memcpy(ct,g->name,g->m*sizeof(char *));
			strcpy(st,s); pvstrlwr(st);
			ct[g->m]=st;
			free(g->name); g->name=ct;
			memcpy(it,g->param,g->m*sizeof(int));
			it[g->m]=v;
			free(g->param); g->param=it;
		}
		else
		{
			char *st=(char *)malloc(strlen(s)+1);
			g->name=(char **)malloc(1*sizeof(char *));
			strcpy(st,s); pvstrlwr(st);
			g->name[0]=st;
			g->param=(int *)malloc(1*sizeof(int));
			g->param[0]=v;
		}
		g->m++;
	}
}

double gvalue(const genome *g,const char *s,const double mn,const double mx)
{
	int n;
	for (n=0;n<g->m;n++) if (!pvstrcmpi(g->name[n],s)) return mn+(double)g->param[n]/(GRES-1)*(mx-mn);
	return 0;
}

int gint(const genome *g,const char *s,const int mn,const int mx)
{
	int n;
	for (n=0;n<g->m;n++) if (!pvstrcmpi(g->name[n],s)) return g->param[n]*(mx-mn+1)/GRES+mn;
	return 0;
}

genome loadgenome(const char *filename,const int offset) // Note: build 3 and above uses generalised 2-line format
{
#ifdef VMS
	int n,r; FILE *in=fopen(filename,"r"); char str[1000];
#else
	int n,r; FILE *in=fopen(filename,"rt"); char str[1000];
#endif
	genome ret=newgenome();
	for (n=r=0;fgets(str,1000,in);)
	{
		char ok=0,*s;
		for (s=str;*s && !ok;s++) if (*s>32) ok=1;
		if (ok)
		{
			r++;
			if (n==offset && r==1) // The line we wanted
			{
				char *pastequals,*pastsemicolon=str;
				for (s=str;*s;s++) // Parse str into ret
				{
					if (*s=='=') pastequals=s+1;
					if (*s==';' && s>pastequals+1)
					{
						char *tmp=(char *)malloc(pastequals-pastsemicolon),
							*tmp2=(char *)malloc(s-pastequals+1);
						int tmpv;
						memcpy(tmp,pastsemicolon,pastequals-pastsemicolon-1);
						tmp[pastequals-pastsemicolon-1]=0;
						memcpy(tmp2,pastequals,s-pastequals);
						tmp2[s-pastequals]=0;
						sscanf(tmp2,"%d",&tmpv);
						gassign(&ret,tmp,tmpv);
						free(tmp); free(tmp2);
						pastsemicolon=s+1;
					}
				}
				fclose(in);
				return ret;
			}
		}
		if (r>=2) {r=0; n++;}
	}
	fclose(in);
	return ret;
}

void savegenome(const genome *g,const double score,const char *comment,const char *filename)
{
#ifdef VMS
	int n; FILE *out=fopen(filename,"a"); char fmtstr[20];
        if (!out) out=fopen(filename,"w");
#else
	int n; FILE *out=fopen(filename,"at"); char fmtstr[20];
	if (!out) out=fopen(filename,"wt");
#endif
	if (!out) return;
	for (n=0;pow(10,n)<GRES;n++);
	sprintf(fmtstr,"%%s=%%0%dd;",n);
	for (n=0;n<g->m;n++) fprintf(out,fmtstr,g->name[n],g->param[n]);
	fprintf(out,"\n%lf %s\n",score,comment);
	fclose(out);
}

double *gscore=NULL; int gm=0;

void readgscores(const char *filename)
{
#ifdef VMS
	int r; FILE *in=fopen(filename,"r"); char str[1000];
#else
	int r; FILE *in=fopen(filename,"rt"); char str[1000];
#endif
	if (!in) {gm=0; gscore=NULL; return;}
	for (gm=r=0;fgets(str,1000,in);)
	{
		char ok=0,*s;
		for (s=str;*s && !ok;s++) if (*s>32) ok=1;
		if (ok) r++;
		if (r>=2) {r=0; gm++;}
	}
	if (gscore) free(gscore);
	if (gm>0)
	{
		int n;
		gscore=(double *)malloc(gm*sizeof(double));
		rewind(in);
		for (n=r=0;fgets(str,1000,in);)
		{
			char ok=0,*s;
			for (s=str;*s && !ok;s++) if (*s>32) ok=1;
			if (ok)
			{
				r++;
				if (r==2) sscanf(str,"%lf",&gscore[n]);
			}
			if (r>=2) {r=0; n++;}
		}
	}
	else gscore=NULL;
	fclose(in);
}

genome fairlygood(const char *filename) // Call readgscores() beforehand
{
	int n,i,bi,samp; double bsc=-1e100;
	if (gm<=0) return newgenome();
	samp=(int)pow(gm,frnd(1.25));
	for (n=0;n<samp;n++)
	{
		i=rnd(gm);
		if (gscore[i]>bsc) {bsc=gscore[i]; bi=i;}
	}
	return loadgenome(filename,bi);
}

///// Main Routine /////

enum TT {Random=0,Mutate=1,Interpolate=2,Crossover=3};

void main(int argc, char **argv)
{
	int n,i; char str[200];
	int m, count = 0;
	particle *p=NULL,*pn,*pm;
	double timesteps=0,dt=0.01e-9;
	char leave=0;
	enum TT trialtype;
	genome curg;
	int parm;


	while ((parm = getopt(argc, argv, "q")) != -1)
	switch(parm)
	{
		case 'q': quiet = 1;
			break;

	}

	srand(time(NULL));
begin_again:;
	// Construct genotype
	curg=newgenome();
	trialtype=(enum TT)rnd(4);
	readgscores(RESULTSFILE);
	if (gm<10) trialtype=Random;
	if (trialtype==Random)
	{
		gassign(&curg,"zsrc",rnd(GRES));
		gassign(&curg,"rotsrc",rnd(GRES));
		gassign(&curg,"d1l",rnd(GRES));
		gassign(&curg,"s2f",rnd(GRES));
		gassign(&curg,"s3f",rnd(GRES));
		gassign(&curg,"s4f",rnd(GRES));
		gassign(&curg,"snf",rnd(GRES));
		gassign(&curg,"alt",rnd(GRES));
		gassign(&curg,"narrow",rnd(GRES));
		gassign(&curg,"na1",rnd(GRES));
		gassign(&curg,"na2",rnd(GRES));
		gassign(&curg,"na3",rnd(GRES));
	}
	else if (trialtype==Mutate)
	{
		genome orig=fairlygood(RESULTSFILE);
		for (n=0;n<orig.m;n++)
			gassign(&curg,orig.name[n],Mini(Maxi((double)orig.param[n]+(int)pow(frnd(2.5)-1.25,25),0),GRES-1));
		removegenome(&orig);
	}
	else if (trialtype==Interpolate)
	{
	
		genome pg1=fairlygood(RESULTSFILE),pg2=fairlygood(RESULTSFILE);
  		float p=frnd(1),q=1.0-p;
		for (n=0;n<pg1.m;n++) for (i=0;i<pg2.m;i++)  if (!pvstrcmpi(pg1.name[n],pg2.name[i]))
			 gassign(&curg,pg1.name[n],(int)(p*pg1.param[n]+q*pg2.param[i]));  
		removegenome(&pg1); removegenome(&pg2);
	}
	else if (trialtype==Crossover)
	{
		genome pg1=fairlygood(RESULTSFILE),pg2=fairlygood(RESULTSFILE); 
		float p=frnd(1);
		for (n=0;n<pg1.m;n++) for (i=0;i<pg2.m;i++) if (!pvstrcmpi(pg1.name[n],pg2.name[i])) 	
			if (frnd(1)<p) gassign(&curg,pg1.name[n],pg1.param[n]); 
			else gassign(&curg,pg1.name[n],pg2.param[i]); 	
		removegenome(&pg1); removegenome(&pg2);
	}
	// Construct phenotype
	{
		double d_d1l,d_s2f,d_s3f,d_s4f,d_snf; int d_alt,d_narrow,d_na1,d_na2,d_na3;
		ZSRC=gvalue(&curg,"zsrc",-0.15,0.15);
		ROTSRC=gvalue(&curg,"rotsrc",0,0.5);
		d_d1l=gvalue(&curg,"d1l",0.5,0.6);
		d_s2f=gvalue(&curg,"s2f",0,5); if (d_s2f<2.5) d_s2f=-(2.5+d_s2f);
		d_s3f=gvalue(&curg,"s3f",0,5); if (d_s3f<2.5) d_s3f=-(2.5+d_s3f);
		d_s4f=gvalue(&curg,"s4f",0,5); if (d_s4f<2.5) d_s4f=-(2.5+d_s4f);
		d_snf=gvalue(&curg,"snf",1,4); if (d_snf<2.5) d_snf=-(1.5+d_snf);
		d_alt=gint(&curg,"alt",0,1);
		d_narrow=gint(&curg,"narrow",0,3);
		d_na1=gint(&curg,"na1",10,30);
		d_na2=gint(&curg,"na2",10,30);
		d_na3=gint(&curg,"na3",10,30);
		if (d_narrow==1 || d_narrow==3) {int low=Mini(d_na1,d_na2); d_na2=Maxi(d_na1,d_na2); d_na1=low;}
		else if (d_narrow==2)
		{
			int low=Mini(d_na1,Mini(d_na2,d_na3)),hig=Maxi(d_na1,Maxi(d_na2,d_na3)),
				mid=d_na1+d_na2+d_na3-low-hig;
			d_na1=low; d_na2=mid; d_na3=hig;
		}
		//ZSRC=0.000; ROTSRC=0.200;
		//d_s2f=d_s4f=-3.30; d_s3f=4.00; d_snf=-3.30; d_alt=1;
		msol=0; // Important: clear space
		addsolenoid(0,0,-0.2033, 0,0,0.2033, 0.1, 20); // Make solenoids
		addsolenoid(0,0,0.2033+d_d1l, 0,0,0.6033+d_d1l, 0.3, d_s2f);
		addsolenoid(0,0,1.1033+d_d1l, 0,0,1.5033+d_d1l, 0.3, d_s3f);
		addsolenoid(0,0,2.0033+d_d1l, 0,0,2.4033+d_d1l, 0.3, d_s4f);
		for (n=5;n<=34;n++)
		{
			double radius;
			if (d_narrow==0) radius=(n>=d_na1?0.15:0.3);
			else if (d_narrow==1) radius=(n>=d_na2?0.15:n>=d_na1?0.225:0.3);
			else if (d_narrow==2) radius=(n>=d_na3?0.15:n>=d_na2?0.2:n>=d_na1?0.25:0.3);
			else if (d_narrow==3) radius=(n>=d_na2?0.15:n>=d_na1?0.3-0.15*(n-d_na1)/Maxi(1,d_na2-d_na1):0.3);
			addsolenoid(
				0,0,0.2033+d_d1l+0.9*(n-2),
				0,0,0.2033+d_d1l+0.9*(n-2)+0.4,
				radius,
				(d_alt?1-n%2*2:1-n/5%2*2)*d_snf);
		}
	}
	t=0; leave=0;
	loadbeam_rodsource("pi_plus_entire",&p,&m);
	pm=p+m;
	cROTSRC=cos(ROTSRC); sROTSRC=sin(ROTSRC);
	count = m;
	while (!leave) // MAIN LOOP //
	{
		if ((int)(timesteps)%100==0)
		{
			sprintf(str, "Simulating to 200 ns... t = %.1f ns, particles left %d",t*1e9, count);
			report(str);
		}
		// Calculation
		count=0;
		for (pn=p;pn<pm;pn++) if (pn->state==Active) advanceparticle(pn,dt), count++;
		t+=dt; timesteps++;
		if ((int)(timesteps)%10==0) for (pn=p;pn<pm;pn++) if (pn->state==Active) findsolenoids(pn);
		if (t>199.999e-9) leave=1;
	}
	double mw=0,pw=0,tw=0;
	for (pn=p;pn<pm;pn++)
	{
		tw+=pn->wt;
		if (pn->state==Finished)
		{
			if (pn->s==Muon) mw+=pn->wt;
			else if (pn->s==Pion) pw+=pn->wt;
		}
	}
	sprintf(str,"(%d particles) [v3.11]",m);
	savegenome(&curg,100.0*mw/tw,str,RESULTSFILE);
	removegenome(&curg);
	sprintf(str,"Simulation finished!  %.3lf%% muon transfer.",100.0*mw/tw);
	report(str);
	goto begin_again;
}

///// Various Disused Bits of Code /////

void diffproc() // Differencing between beams
{
	int n,i;
#ifdef VMS
	FILE *out=fopen(".\\diff_z42.txt","w");
#else
	FILE *out=fopen(".\\diff_z42.txt","wt");
#endif
	char *files[9]={"ef_inf","ef_10","ef_5","ef_3","ef_2","ef_1_5","ef_1","ef_0_5","ef_0"};
	fprintf(out,"Distance: Max [99%% 90%%] RMS Mean\n");
	if (0) for (n=1;n<9;n++)
	{
		i=0;
		fprintf(out,"%s-%s %lf %lf %lf\n",files[n],files[i],
			diff_infinitynorm(files[n],files[i]),
			diff_rms(files[n],files[i]),
			diff_mean(files[n],files[i]));
	}
	fprintf(out,"\neef_on-eef_off %.15lf %.15lf %.15lf\n",
		diff_infinitynorm("results\\eef_on","results\\eef_off"),
		diff_rms("results\\eef_on","results\\eef_off"),
		diff_mean("results\\eef_on","results\\eef_off"));
	fprintf(out,"\neef_on-eef_onp3 %.15lf %.15lf %.15lf\n",
		diff_infinitynorm("results\\eef_on","results\\eef_onp3"),
		diff_rms("results\\eef_on","results\\eef_onp3"),
		diff_mean("results\\eef_on","results\\eef_onp3"));
	fprintf(out,"\neef_off-eef_onp3 %.15lf %.15lf %.15lf\n",
		diff_infinitynorm("results\\eef_off","results\\eef_onp3"),
		diff_rms("results\\eef_off","results\\eef_onp3"),
		diff_mean("results\\eef_off","results\\eef_onp3"));
	fclose(out);
}

void bfieldproc() // Testing B field along solenoid
{/*
#ifdef VMS
	double rs,z,r=RSOL/2; FILE *out=fopen(".\\Bfield.txt","w");
#else
	double rs,z,r=RSOL/2; FILE *out=fopen(".\\Bfield.txt","wt");
#endif
	fprintf(out,"RSOL = %lf\n(z, r) (Bz, Br) zorder/rorder\n",rs=RSOL);
	for (z=-10;z<LSOL+10;z++)
		fprintf(out,"(%lf, %lf)  (%.9lf, %.9lf) %d/%d\n",z,r,bz(z,r),br(z,r),zorder,rorder);
	r=0; fprintf(out,"\n");
	for (z=-10;z<LSOL+10;z++)
		fprintf(out,"(%lf, %lf)  (%.9lf, %.9lf) %d/%d\n",z,r,bz(z,r),br(z,r),zorder,rorder);
	r=RSOL*0.9; fprintf(out,"\n");
	for (z=-10;z<LSOL+10;z++)
		fprintf(out,"(%lf, %lf)  (%.9lf, %.9lf) %d/%d\n",z,r,bz(z,r),br(z,r),zorder,rorder);
	fclose(out);
	return 0;*///Fix all this later
}

void csvoutproc(particle *p,const int m) // Output particle cords as CSV file
{
	FILE *out; char str[200]; particle *pn,*pm=p+m;
	sprintf(str,".\\out%dmrad.csv",floor(ROTSRC*1000+0.5));
#ifdef VMS
	out=fopen(str,"w");
#else
	out=fopen(str,"wt");
#endif
	fprintf(out,"particle weight, x,y,z (metres), vx,vy,vz (m/s)\n");
	for (pn=p;pn<pm;pn++) if (pn->state==Active)
		fprintf(out,"%lf, %lf,%lf,%lf, %lf,%lf,%lf\n",
			pn->wt,pn->x,pn->y,pn->z,pn->vx,pn->vy,pn->vz);
	fclose(out);
}
